package com.muyu.utils;
import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.RunInstancesRequest;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import lombok.SneakyThrows;
import okhttp3.*;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;


/**
 * ECS工具类，提供创建ECS实例的功能
 */
public class ECSTool {
    /**
     * 创建ECS客户端
     * @return ECS客户端实例
     * @throws Exception 如果配置信息不正确或网络问题，将抛出异常
     * 注意：此方法用于演示，实际使用时请替换为安全的鉴权方式，如STS
     * 更多鉴权访问方式参考：https://help.aliyun.com/document_detail/378657.html
     */
    public static Client createClient() throws Exception {
        // 配置ECS客户端的基本信息，包括访问密钥和endpoint
        Config config = new Config()
                .setAccessKeyId("LTAI5tPTk3MFkmCGBbnQgmrM") // 示例Access Key ID，实际使用时应从环境变量获取
                .setAccessKeySecret("q7rLjxrI0SLBXlvNT4VmYcHCNCY2p6"); // 示例Access Key Secret，实际使用时应从环境变量获取
        config.endpoint = "ecs.cn-zhangjiakou.aliyuncs.com";  // 设置ECS服务的访问地址
        // 创建并返回ECS客户端实例
        return new Client(config);
    }

    /**
     * 创建并运行ECS实例
     * @param regionId 地域ID
     * @param launchTemplateId 启动模板ID
     * @throws Exception 如果调用API时发生错误，将抛出异常
     */
    public static void runEcsInstance(String regionId, String launchTemplateId) throws Exception {
        // 创建ECS客户端
        Client client = ECSTool.createClient();
        // 设置运行实例的请求参数
        RunInstancesRequest runInstancesRequest = new RunInstancesRequest()
                .setRegionId(regionId)
                .setLaunchTemplateId(launchTemplateId);
        // 创建运行选项
        RuntimeOptions runtime = new RuntimeOptions();

        try {
            // 调用API运行实例
            client.runInstancesWithOptions(runInstancesRequest, runtime);
        } catch (Exception error) {
            // 处理API调用过程中出现的异常
            System.out.println(error.getMessage());
            if (error instanceof TeaException) {
                // 处理特定类型的异常，如TeaException
                TeaException teaError = (TeaException) error;
                System.out.println(teaError.getData().get("Recommend")); // 打印诊断推荐链接
                com.aliyun.teautil.Common.assertAsString(teaError.getMessage()); // 断言错误信息
            } else {
                // 处理其他类型的异常
                System.out.println(error.getMessage());
            }
        }
    }
    public static void main(String[] args) throws Exception {
        List<String> argsList = Arrays.asList(args);
        // 示例调用，实际使用时需要传入  区域 ID  和  启动模板 ID
        runEcsInstance("cn-zhangjiakou", "lt-8vbepqjihmawbkqcwkcm");
    }

//    @SneakyThrows
//    public static void main(String[] args)  {
//            OkHttpClient client = new OkHttpClient.Builder()
//                    .build();
//
//            // 注意：GET请求通常不需要MediaType和RequestBody
//            Request request = new Request.Builder()
//                    .url("http://fluxmq.muyu.icu/public/cluster")
//                    .method("GET", null) // 对于GET请求，通常不需要设置RequestBody，所以传入null
//                    .addHeader("User-Agent", "Apifox/1.0.0 (https://apifox.com)")
//                    .addHeader("Access-Token", "2f68dbbf-519d-4f01-9636-e2421b68f379") // 确保使用正确的头部名称和令牌值
//                    .build();
//
//            try (Response response = client.newCall(request).execute()) {
//                if (!response.isSuccessful()) {
//                    throw new IOException("Unexpected code " + response);
//                } else {
//                    // 打印响应体
//                    System.out.println(response );
//                }
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        }
}
