package com.zhiLian.controller;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.zhiLian.common.Result;
import com.zhiLian.common.ConnectWeight;
import com.zhiLian.service.LoadBalanceService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;


import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @ClassName LoadBalanceController
 * @Description 描述
 * @Author YunFei.Du
 * @Date 2024/5/28 9:12
 */
@RestController
@RequestMapping("loadBalance")
@Log4j2
public class LoadBalanceController {


    private static List< String > ipListThread = new ArrayList<> ( );

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private LoadBalanceService loadBalanceService;

    /**
     * 创建实例
     */
    @GetMapping("/createConnect")
    public void createConnect() {
        loadBalanceService.createConnect ( );
        log.info ( "创建实例成功" );
    }

//    /**
//     * 停止实例
//     */
//    @GetMapping("/stopConnect")
//    public void stopConnect(@RequestParam String instanceId) {
//        loadBalanceService.stopConnect(instanceId);
//        log.info("停止实例成功");
//    }

    /**
     * 销毁实例
     * instanceId  实例id
     */
    @GetMapping("/removeConnect")
    public void removeConnect(@RequestParam String instanceId) {
        loadBalanceService.removeConnect ( instanceId );
        log.info ( "销毁实例成功" );
    }

    /**
     * 获取实例连接
     */
    @GetMapping("/getIpList")
    public Result< List< String > > getIpList() {
        List< String > ipList = loadBalanceService.getIpList ( );
        ipListThread=ipList;
        return Result.success ( ipList );
    }

    /**
     * 获取token
     */
    @PostMapping("/getToken")
    public void getToken() {

        List< ConnectWeight > connectWeightList = new ArrayList<> ( );

        for (String ip : ipListThread) {
            // 向服务器发送登录请求
            String url = "http://" + ip + ":8080/public/login";
            Map< String, Object > request = new HashMap<> ( );
            request.put ( "username", "fluxmq" );
            request.put ( "password", "fluxmq" );
            HttpHeaders httpHeaders = new HttpHeaders ( );
            httpHeaders.setContentType ( MediaType.APPLICATION_JSON );
            HttpEntity< Map< String, Object > > r = new HttpEntity< Map< String, Object > > ( request, httpHeaders );
            String result = restTemplate.postForObject ( url, r, String.class );
            // 添加随机参数以避免缓存
            int nextInt = new Random ( ).nextInt ( 1000 );
            String getInfoUrl = "http://" + ip + ":8080/public/cluster?random=" + nextInt;
            // 使用获取到的token向服务器查询当前连接数
            HttpHeaders httpHeadersGetInfo = new HttpHeaders ( );
            httpHeadersGetInfo.setContentType ( MediaType.APPLICATION_JSON );
            httpHeadersGetInfo.setAccept ( Collections.singletonList ( MediaType.APPLICATION_JSON ) );
            httpHeadersGetInfo.set ( "Cookie", result );
            HttpEntity getInfoRequest = new HttpEntity ( httpHeadersGetInfo );
            ResponseEntity< String > responseInfo = restTemplate.exchange ( getInfoUrl, HttpMethod.GET, getInfoRequest, String.class, 1 );

            // 将字符串转换为有效的JSON格式

            JSONArray jsonArray = JSON.parseArray ( responseInfo.getBody ( ) );
            log.info ( "响应是" + jsonArray );

            if (jsonArray.size ( ) > 0) {

                JSONObject jsonObject = jsonArray.getJSONObject ( 0 );
                Integer connectSize = Integer.valueOf ( jsonObject.getJSONObject ( "mqttInfo" ).getString ( "connectSize" ) );
                connectWeightList.add ( new ConnectWeight ( ip, 100 - connectSize ) );
                log.error ( "链接数量:{}", connectSize );
            } else {
                log.error ( "得到的相应数据为null" );
            }
        }
        Integer sum = 0;
        for (ConnectWeight connectWeight : connectWeightList) {
            sum = sum + connectWeight.getWeightValue ( );
        }

        int max = 0;
        for (ConnectWeight connectWeight : connectWeightList) {
            log.error ( "权重值:{}", connectWeight.getWeightValue ( ) );
            Integer result = BigDecimal.valueOf ( connectWeight.getWeightValue ( ) * 100 ).divide ( BigDecimal.valueOf ( sum ), 0, RoundingMode.DOWN ).intValue ( );
            if (result > max) {
                max = result;
            }
            connectWeight.setWeightValue ( result );
            log.error ( "100次轮询次数:{}", result );
        }

        ArrayList< String > weightIpList = new ArrayList<> ( );

        //轮询出现次数
        for (int i = 0; i <= max; i++) {
            for (ConnectWeight connectWeight : connectWeightList) {
                if (connectWeight.getWeightValue ( ) > i) {
                    weightIpList.add ( connectWeight.getCarServerIp ( ) );
                } else if (connectWeight.getWeightValue ( ) == max) {
                    weightIpList.add ( connectWeight.getCarServerIp ( ) );
                }
            }
        }
        //存入redis
        redisTemplate.delete ( "ipList" );
        for (String ip : weightIpList) {
            redisTemplate.opsForList ( ).rightPush ( "ipList", ip );
        }
    }

}
