package com.zhiLian.resp;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
  *  连接响应
  * @author YunFei.Du
  * @date  9:10 2024/5/27
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class ConnectResp {

    /**
     * 节点名称
     */
    private String nodeName;
    /**
     * 节点版本
     */
    private String version;
    /**
     * 启动时间
     */
    private String startJvmTime;
    /**
     * 节点ID
     */
    private String clusterId;
    /**
     * HTTP请求地址
     */
    private String httpUrl;
    /**
     * MQTTS请求地址
     */
    private String mqttUrl;
    /**
     * websocket请求地址
     */
    private String websocketUrl;


    private FlowInfo flowInfo;


    /**
     * 节点状态
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    private static class FlowInfo {
        /**
         * 写入总吞吐量
         */
        private String writeBytesHistory;
        /**
         * 读取总吞吐量
         */
        private String readBytesHistory;
        /**
         * 上次读取吞吐量
         */
        private String lastReadThroughput;
        /**
         * 上次写入吞吐量
         */
        private String lastWriteThroughput;
        /**
         * 实写字节
         */
        private String realWriteBytes;
    }

    public CpuInfo cpuInfo;


    /**
     * cpu使用信息
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    public static class CpuInfo {
        /**
         * CPU使用信息
         */
        private String cSys;
        /**
         * 空闲率
         */
        private String idle;
        /**
         * I/O等待
         */
        private String iowait;
        /**
         * 用户态使用率
         */
        private String user;
        /**
         * CPU核数
         */
        private Integer cpuNum;
    }


    public JvmInfo jvmInfo;

    /**
     * jvm使用信息
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    public static class JvmInfo {
        /**
         * JAVA目录
         */
        private String jdkHome;
        /**
         * 堆内存
         */
        private String heapCommit;
        /**
         * 堆最大内存
         */
        private String heapMax;
        /**
         * 线程数量
         */
        private Integer threadCount;
        /**
         * 文件描述(句柄
         */
        private String fileDescriptors;
        /**
         * 非堆最大空间
         */
        private String noHeapMax;
        /**
         * 非堆初始化空间
         */
        private String noHeapInit;
        /**
         * JDK版本
         */
        private String jdkVersion;
        /**
         * 堆初始化空间
         */
        private String heapInit;

        /**
         * 堆使用空间
         */
        private String heapUsed;
        /**
         * 非堆空间
         */
        private String noHeapCommit;
        /**
         * 非堆使用空间
         */
        private String noHeapUsed;

    }


    public MqttInfo mqttInfo;

    /**
     * MQTT事件信息
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    public static class MqttInfo {
        /**
         * 连接事件数量
         */
        private Integer connectEventSize;
        /**
         * 订阅事件数量
         */
        private Integer subscribeEventSize;
        /**
         * 发布重试事件数量
         */
        private Integer publishRetryEventSize;
        /**
         * 主题数量
         */
        private Integer topicSize;
        /**
         * 断开链接数量
         */
        private Integer disconnectEventSize;
        /**
         * 订阅数量
         */
        private Integer subscribeSize;
        /**
         * 推送数量
         */
        private Integer publishEventSize;
        /**
         * 关闭事件数量
         */
        private Integer closeEventSize;
        /**
         * 链接总数
         */
        private Integer connectSize;
        /**
         * 保留消息数量
         */
        private Integer retainSize;
        /**
         * 取消订阅数量
         */
        private Integer unSubscribeEventSize;
    }


}
